/* valaccodetypedefinition.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodetypedefinition.h>
#include <ccode/valaccodelinedirective.h>




struct _ValaCCodeTypeDefinitionPrivate {
	char* _type_name;
	ValaCCodeDeclarator* _declarator;
};

#define VALA_CCODE_TYPE_DEFINITION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_TYPE_DEFINITION, ValaCCodeTypeDefinitionPrivate))
enum  {
	VALA_CCODE_TYPE_DEFINITION_DUMMY_PROPERTY,
	VALA_CCODE_TYPE_DEFINITION_TYPE_NAME,
	VALA_CCODE_TYPE_DEFINITION_DECLARATOR
};
static void vala_ccode_type_definition_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_type_definition_real_write_declaration (ValaCCodeNode* base, ValaCCodeWriter* writer);
static gpointer vala_ccode_type_definition_parent_class = NULL;
static void vala_ccode_type_definition_dispose (GObject * obj);



ValaCCodeTypeDefinition* vala_ccode_type_definition_new (const char* type, ValaCCodeDeclarator* decl) {
	ValaCCodeTypeDefinition * self;
	g_return_val_if_fail (type != NULL, NULL);
	g_return_val_if_fail (VALA_IS_CCODE_DECLARATOR (decl), NULL);
	self = g_object_newv (VALA_TYPE_CCODE_TYPE_DEFINITION, 0, NULL);
	vala_ccode_type_definition_set_type_name (self, type);
	vala_ccode_type_definition_set_declarator (self, decl);
	return self;
}


static void vala_ccode_type_definition_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeTypeDefinition * self;
	self = VALA_CCODE_TYPE_DEFINITION (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
}


static void vala_ccode_type_definition_real_write_declaration (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeTypeDefinition * self;
	self = VALA_CCODE_TYPE_DEFINITION (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_indent (writer, NULL);
	vala_ccode_writer_write_string (writer, "typedef ");
	vala_ccode_writer_write_string (writer, self->priv->_type_name);
	vala_ccode_writer_write_string (writer, " ");
	vala_ccode_node_write_declaration (VALA_CCODE_NODE (self->priv->_declarator), writer);
	vala_ccode_writer_write_string (writer, ";");
	vala_ccode_writer_write_newline (writer);
}


char* vala_ccode_type_definition_get_type_name (ValaCCodeTypeDefinition* self) {
	g_return_val_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self), NULL);
	return self->priv->_type_name;
}


void vala_ccode_type_definition_set_type_name (ValaCCodeTypeDefinition* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_type_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_type_name = (g_free (self->priv->_type_name), NULL)), _tmp2);
}


ValaCCodeDeclarator* vala_ccode_type_definition_get_declarator (ValaCCodeTypeDefinition* self) {
	g_return_val_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self), NULL);
	return self->priv->_declarator;
}


void vala_ccode_type_definition_set_declarator (ValaCCodeTypeDefinition* self, ValaCCodeDeclarator* value) {
	ValaCCodeDeclarator* _tmp2;
	ValaCCodeDeclarator* _tmp1;
	g_return_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_declarator = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_declarator == NULL ? NULL : (self->priv->_declarator = (g_object_unref (self->priv->_declarator), NULL))), _tmp2);
}


static void vala_ccode_type_definition_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaCCodeTypeDefinition * self;
	self = VALA_CCODE_TYPE_DEFINITION (object);
	switch (property_id) {
		case VALA_CCODE_TYPE_DEFINITION_TYPE_NAME:
		g_value_set_string (value, vala_ccode_type_definition_get_type_name (self));
		break;
		case VALA_CCODE_TYPE_DEFINITION_DECLARATOR:
		g_value_set_object (value, vala_ccode_type_definition_get_declarator (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_type_definition_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaCCodeTypeDefinition * self;
	self = VALA_CCODE_TYPE_DEFINITION (object);
	switch (property_id) {
		case VALA_CCODE_TYPE_DEFINITION_TYPE_NAME:
		vala_ccode_type_definition_set_type_name (self, g_value_get_string (value));
		break;
		case VALA_CCODE_TYPE_DEFINITION_DECLARATOR:
		vala_ccode_type_definition_set_declarator (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_type_definition_class_init (ValaCCodeTypeDefinitionClass * klass) {
	vala_ccode_type_definition_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCCodeTypeDefinitionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_type_definition_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_type_definition_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_type_definition_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_type_definition_real_write;
	VALA_CCODE_NODE_CLASS (klass)->write_declaration = vala_ccode_type_definition_real_write_declaration;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_TYPE_DEFINITION_TYPE_NAME, g_param_spec_string ("type-name", "type-name", "type-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_TYPE_DEFINITION_DECLARATOR, g_param_spec_object ("declarator", "declarator", "declarator", VALA_TYPE_CCODE_DECLARATOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_ccode_type_definition_init (ValaCCodeTypeDefinition * self) {
	self->priv = VALA_CCODE_TYPE_DEFINITION_GET_PRIVATE (self);
}


static void vala_ccode_type_definition_dispose (GObject * obj) {
	ValaCCodeTypeDefinition * self;
	self = VALA_CCODE_TYPE_DEFINITION (obj);
	self->priv->_type_name = (g_free (self->priv->_type_name), NULL);
	(self->priv->_declarator == NULL ? NULL : (self->priv->_declarator = (g_object_unref (self->priv->_declarator), NULL)));
	G_OBJECT_CLASS (vala_ccode_type_definition_parent_class)->dispose (obj);
}


GType vala_ccode_type_definition_get_type (void) {
	static GType vala_ccode_type_definition_type_id = 0;
	if (G_UNLIKELY (vala_ccode_type_definition_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeTypeDefinitionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_type_definition_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeTypeDefinition), 0, (GInstanceInitFunc) vala_ccode_type_definition_init };
		vala_ccode_type_definition_type_id = g_type_register_static (VALA_TYPE_CCODE_NODE, "ValaCCodeTypeDefinition", &g_define_type_info, 0);
	}
	return vala_ccode_type_definition_type_id;
}




