/* valacfgbuilder.vala
 *
 * Copyright (C) 2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valacfgbuilder.h>
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gee/collection.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valabasicblock.h>
#include <vala/valavoidtype.h>
#include <vala/valareport.h>
#include <vala/valacodenode.h>
#include <vala/valasourcereference.h>
#include <vala/valalocalvariable.h>
#include <vala/valainvocationexpression.h>
#include <vala/valamemberaccess.h>
#include <vala/valaexpression.h>
#include <vala/valaswitchsection.h>
#include <vala/valastatement.h>
#include <vala/valacatchclause.h>
#include <vala/valadatatype.h>
#include <vala/valaerrorcode.h>


#define VALA_CFG_BUILDER_TYPE_JUMP_TARGET (vala_cfg_builder_jump_target_get_type ())
#define VALA_CFG_BUILDER_JUMP_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ValaCFGBuilderJumpTarget))
#define VALA_CFG_BUILDER_JUMP_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ValaCFGBuilderJumpTargetClass))
#define VALA_CFG_BUILDER_IS_JUMP_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_CFG_BUILDER_TYPE_JUMP_TARGET))
#define VALA_CFG_BUILDER_IS_JUMP_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_CFG_BUILDER_TYPE_JUMP_TARGET))
#define VALA_CFG_BUILDER_JUMP_TARGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ValaCFGBuilderJumpTargetClass))

typedef struct _ValaCFGBuilderJumpTarget ValaCFGBuilderJumpTarget;
typedef struct _ValaCFGBuilderJumpTargetClass ValaCFGBuilderJumpTargetClass;
typedef struct _ValaCFGBuilderJumpTargetPrivate ValaCFGBuilderJumpTargetPrivate;

struct _ValaCFGBuilderJumpTarget {
	GObject parent_instance;
	ValaCFGBuilderJumpTargetPrivate * priv;
};

struct _ValaCFGBuilderJumpTargetClass {
	GObjectClass parent_class;
};



struct _ValaCFGBuilderPrivate {
	ValaCodeContext* context;
	ValaBasicBlock* current_block;
	gboolean unreachable_reported;
	GeeList* jump_stack;
};

#define VALA_CFG_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CFG_BUILDER, ValaCFGBuilderPrivate))
enum  {
	VALA_CFG_BUILDER_DUMMY_PROPERTY
};
static void vala_cfg_builder_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_cfg_builder_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_cfg_builder_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_cfg_builder_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_cfg_builder_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_cfg_builder_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed);
static void vala_cfg_builder_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_cfg_builder_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_cfg_builder_real_visit_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_cfg_builder_real_visit_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_cfg_builder_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt);
static void vala_cfg_builder_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_cfg_builder_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_cfg_builder_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt);
static void vala_cfg_builder_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_cfg_builder_real_visit_do_statement (ValaCodeVisitor* base, ValaDoStatement* stmt);
static void vala_cfg_builder_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_cfg_builder_real_visit_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_cfg_builder_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt);
static void vala_cfg_builder_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt);
static void vala_cfg_builder_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_cfg_builder_handle_errors (ValaCFGBuilder* self, ValaCodeNode* node);
static void vala_cfg_builder_real_visit_throw_statement (ValaCodeVisitor* base, ValaThrowStatement* stmt);
static void vala_cfg_builder_real_visit_try_statement (ValaCodeVisitor* base, ValaTryStatement* stmt);
static void vala_cfg_builder_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt);
static gboolean vala_cfg_builder_unreachable (ValaCFGBuilder* self, ValaCodeNode* node);
struct _ValaCFGBuilderJumpTargetPrivate {
	gboolean _break_target;
	gboolean _continue_target;
	gboolean _return_target;
	gboolean _error_target;
	ValaErrorDomain* _error_domain;
	ValaErrorCode* _error_code;
	gboolean _finally_clause;
	ValaBasicBlock* _basic_block;
	ValaBasicBlock* _last_block;
	ValaCatchClause* _catch_clause;
};

#define VALA_CFG_BUILDER_JUMP_TARGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ValaCFGBuilderJumpTargetPrivate))
enum  {
	VALA_CFG_BUILDER_JUMP_TARGET_DUMMY_PROPERTY,
	VALA_CFG_BUILDER_JUMP_TARGET_BREAK_TARGET,
	VALA_CFG_BUILDER_JUMP_TARGET_CONTINUE_TARGET,
	VALA_CFG_BUILDER_JUMP_TARGET_RETURN_TARGET,
	VALA_CFG_BUILDER_JUMP_TARGET_ERROR_TARGET,
	VALA_CFG_BUILDER_JUMP_TARGET_ERROR_DOMAIN,
	VALA_CFG_BUILDER_JUMP_TARGET_ERROR_CODE,
	VALA_CFG_BUILDER_JUMP_TARGET_FINALLY_CLAUSE,
	VALA_CFG_BUILDER_JUMP_TARGET_BASIC_BLOCK,
	VALA_CFG_BUILDER_JUMP_TARGET_LAST_BLOCK,
	VALA_CFG_BUILDER_JUMP_TARGET_CATCH_CLAUSE
};
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_break_target (ValaBasicBlock* basic_block);
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_continue_target (ValaBasicBlock* basic_block);
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_return_target (ValaBasicBlock* basic_block);
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_error_target (ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code);
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_finally_clause (ValaBasicBlock* basic_block, ValaBasicBlock* last_block);
static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new (void);
static gboolean vala_cfg_builder_jump_target_get_break_target (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_break_target (ValaCFGBuilderJumpTarget* self, gboolean value);
static gboolean vala_cfg_builder_jump_target_get_continue_target (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_continue_target (ValaCFGBuilderJumpTarget* self, gboolean value);
static gboolean vala_cfg_builder_jump_target_get_return_target (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_return_target (ValaCFGBuilderJumpTarget* self, gboolean value);
static gboolean vala_cfg_builder_jump_target_get_error_target (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_error_target (ValaCFGBuilderJumpTarget* self, gboolean value);
static ValaErrorDomain* vala_cfg_builder_jump_target_get_error_domain (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_error_domain (ValaCFGBuilderJumpTarget* self, ValaErrorDomain* value);
static ValaErrorCode* vala_cfg_builder_jump_target_get_error_code (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_error_code (ValaCFGBuilderJumpTarget* self, ValaErrorCode* value);
static gboolean vala_cfg_builder_jump_target_get_finally_clause (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_finally_clause (ValaCFGBuilderJumpTarget* self, gboolean value);
static ValaBasicBlock* vala_cfg_builder_jump_target_get_basic_block (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_basic_block (ValaCFGBuilderJumpTarget* self, ValaBasicBlock* value);
static ValaBasicBlock* vala_cfg_builder_jump_target_get_last_block (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_last_block (ValaCFGBuilderJumpTarget* self, ValaBasicBlock* value);
static ValaCatchClause* vala_cfg_builder_jump_target_get_catch_clause (ValaCFGBuilderJumpTarget* self);
static void vala_cfg_builder_jump_target_set_catch_clause (ValaCFGBuilderJumpTarget* self, ValaCatchClause* value);
static gpointer vala_cfg_builder_jump_target_parent_class = NULL;
static void vala_cfg_builder_jump_target_dispose (GObject * obj);
static GType vala_cfg_builder_jump_target_get_type (void);
static gpointer vala_cfg_builder_parent_class = NULL;
static void vala_cfg_builder_dispose (GObject * obj);



ValaCFGBuilder* vala_cfg_builder_new (void) {
	ValaCFGBuilder * self;
	self = g_object_newv (VALA_TYPE_CFG_BUILDER, 0, NULL);
	return self;
}


/**
 * Build control flow graph in the specified context.
 *
 * @param context a code context
 */
void vala_cfg_builder_build_cfg (ValaCFGBuilder* self, ValaCodeContext* context) {
	ValaCodeContext* _tmp1;
	ValaCodeContext* _tmp0;
	GeeCollection* source_files;
	g_return_if_fail (VALA_IS_CFG_BUILDER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->context = (_tmp1 = (_tmp0 = context, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL))), _tmp1);
	/* we're only interested in non-pkg source files */
	source_files = vala_code_context_get_source_files (context);
	{
		GeeCollection* file_collection;
		GeeIterator* file_it;
		file_collection = source_files;
		file_it = gee_iterable_iterator (GEE_ITERABLE (file_collection));
		while (gee_iterator_next (file_it)) {
			ValaSourceFile* file;
			file = ((ValaSourceFile*) gee_iterator_get (file_it));
			{
				if (!vala_source_file_get_external_package (file)) {
					vala_source_file_accept (file, VALA_CODE_VISITOR (self));
				}
				(file == NULL ? NULL : (file = (g_object_unref (file), NULL)));
			}
		}
		(file_it == NULL ? NULL : (file_it = (g_object_unref (file_it), NULL)));
	}
	(source_files == NULL ? NULL : (source_files = (g_object_unref (source_files), NULL)));
}


static void vala_cfg_builder_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	vala_source_file_accept_children (source_file, VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	vala_code_node_accept_children (VALA_CODE_NODE (cl), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	vala_code_node_accept_children (VALA_CODE_NODE (st), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	vala_code_node_accept_children (VALA_CODE_NODE (iface), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	vala_code_node_accept_children (VALA_CODE_NODE (en), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_ERROR_DOMAIN (ed));
	vala_code_node_accept_children (VALA_CODE_NODE (ed), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	ValaCFGBuilder * self;
	ValaBasicBlock* _tmp0;
	ValaBasicBlock* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaCFGBuilderJumpTarget* _tmp3;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_method_get_body (m) == NULL) {
		return;
	}
	_tmp0 = NULL;
	vala_method_set_entry_block (m, (_tmp0 = vala_basic_block_new_entry ()));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	_tmp1 = NULL;
	vala_method_set_exit_block (m, (_tmp1 = vala_basic_block_new_exit ()));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	_tmp2 = NULL;
	self->priv->current_block = (_tmp2 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
	vala_basic_block_connect (vala_method_get_entry_block (m), self->priv->current_block);
	_tmp3 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp3 = vala_cfg_builder_jump_target_new_return_target (vala_method_get_exit_block (m))));
	(_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL)));
	vala_code_node_accept_children (VALA_CODE_NODE (m), VALA_CODE_VISITOR (self));
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	if (self->priv->current_block != NULL) {
		/* end of method body reachable*/
		if (!(VALA_IS_VOID_TYPE (vala_method_get_return_type (m)))) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), "missing return statement at end of method body");
			vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
		}
		vala_basic_block_connect (self->priv->current_block, vala_method_get_exit_block (m));
	}
}


static void vala_cfg_builder_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	vala_code_node_accept_children (VALA_CODE_NODE (prop), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc) {
	ValaCFGBuilder * self;
	ValaBasicBlock* _tmp0;
	ValaBasicBlock* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaCFGBuilderJumpTarget* _tmp3;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	if (vala_property_accessor_get_body (acc) == NULL) {
		return;
	}
	_tmp0 = NULL;
	vala_property_accessor_set_entry_block (acc, (_tmp0 = vala_basic_block_new_entry ()));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	_tmp1 = NULL;
	vala_property_accessor_set_exit_block (acc, (_tmp1 = vala_basic_block_new_exit ()));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	_tmp2 = NULL;
	self->priv->current_block = (_tmp2 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
	vala_basic_block_connect (vala_property_accessor_get_entry_block (acc), self->priv->current_block);
	_tmp3 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp3 = vala_cfg_builder_jump_target_new_return_target (vala_property_accessor_get_exit_block (acc))));
	(_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL)));
	vala_code_node_accept_children (VALA_CODE_NODE (acc), VALA_CODE_VISITOR (self));
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	if (self->priv->current_block != NULL) {
		/* end of property accessor body reachable*/
		if (vala_property_accessor_get_readable (acc)) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (acc)), "missing return statement at end of property getter body");
			vala_code_node_set_error (VALA_CODE_NODE (acc), TRUE);
		}
		vala_basic_block_connect (self->priv->current_block, vala_property_accessor_get_exit_block (acc));
	}
}


static void vala_cfg_builder_real_visit_block (ValaCodeVisitor* base, ValaBlock* b) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	vala_code_node_accept_children (VALA_CODE_NODE (b), VALA_CODE_VISITOR (self));
}


static void vala_cfg_builder_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt) {
	ValaCFGBuilder * self;
	ValaLocalVariable* _tmp1;
	ValaSymbol* _tmp0;
	ValaLocalVariable* local;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_DECLARATION_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	_tmp1 = NULL;
	_tmp0 = NULL;
	local = (_tmp1 = (_tmp0 = vala_declaration_statement_get_declaration (stmt), (VALA_IS_LOCAL_VARIABLE (_tmp0) ? ((ValaLocalVariable*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	if (local != NULL && vala_local_variable_get_initializer (local) != NULL) {
		vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_local_variable_get_initializer (local)));
	}
	(local == NULL ? NULL : (local = (g_object_unref (local), NULL)));
}


static void vala_cfg_builder_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (stmt));
	if (VALA_IS_INVOCATION_EXPRESSION (vala_expression_statement_get_expression (stmt))) {
		ValaInvocationExpression* _tmp0;
		ValaInvocationExpression* expr;
		ValaMemberAccess* _tmp2;
		ValaExpression* _tmp1;
		ValaMemberAccess* ma;
		ValaAttribute* _tmp3;
		gboolean _tmp4;
		_tmp0 = NULL;
		expr = (_tmp0 = VALA_INVOCATION_EXPRESSION (vala_expression_statement_get_expression (stmt)), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
		_tmp2 = NULL;
		_tmp1 = NULL;
		ma = (_tmp2 = (_tmp1 = vala_invocation_expression_get_call (expr), (VALA_IS_MEMBER_ACCESS (_tmp1) ? ((ValaMemberAccess*) _tmp1) : NULL)), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
		_tmp3 = NULL;
		if ((_tmp4 = vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)) != NULL && (_tmp3 = vala_code_node_get_attribute (VALA_CODE_NODE (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))), "NoReturn")) != NULL, (_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL))), _tmp4)) {
			ValaBasicBlock* _tmp5;
			_tmp5 = NULL;
			self->priv->current_block = (_tmp5 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp5);
			self->priv->unreachable_reported = FALSE;
			(expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL)));
			(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
			return;
		}
		(expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL)));
		(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
	}
}


static void vala_cfg_builder_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* _tmp0;
	ValaBasicBlock* last_block;
	ValaBasicBlock* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* last_true_block;
	ValaBasicBlock* _tmp3;
	ValaBasicBlock* _tmp4;
	ValaBasicBlock* last_false_block;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	/* condition*/
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (vala_if_statement_get_condition (stmt)));
	vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_if_statement_get_condition (stmt)));
	/* true block*/
	_tmp0 = NULL;
	last_block = (_tmp0 = self->priv->current_block, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	_tmp1 = NULL;
	self->priv->current_block = (_tmp1 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp1);
	vala_basic_block_connect (last_block, self->priv->current_block);
	vala_code_node_accept (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)), VALA_CODE_VISITOR (self));
	/* false block*/
	_tmp2 = NULL;
	last_true_block = (_tmp2 = self->priv->current_block, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	_tmp3 = NULL;
	self->priv->current_block = (_tmp3 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp3);
	vala_basic_block_connect (last_block, self->priv->current_block);
	if (vala_if_statement_get_false_statement (stmt) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_if_statement_get_false_statement (stmt)), VALA_CODE_VISITOR (self));
	}
	/* after if/else*/
	_tmp4 = NULL;
	last_false_block = (_tmp4 = self->priv->current_block, (_tmp4 == NULL ? NULL : g_object_ref (_tmp4)));
	/* reachable?*/
	if (last_true_block != NULL || last_false_block != NULL) {
		ValaBasicBlock* _tmp5;
		_tmp5 = NULL;
		self->priv->current_block = (_tmp5 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp5);
		if (last_true_block != NULL) {
			vala_basic_block_connect (last_true_block, self->priv->current_block);
		}
		if (last_false_block != NULL) {
			vala_basic_block_connect (last_false_block, self->priv->current_block);
		}
	}
	(last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL)));
	(last_true_block == NULL ? NULL : (last_true_block = (g_object_unref (last_true_block), NULL)));
	(last_false_block == NULL ? NULL : (last_false_block = (g_object_unref (last_false_block), NULL)));
}


static void vala_cfg_builder_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* after_switch_block;
	ValaCFGBuilderJumpTarget* _tmp0;
	ValaBasicBlock* _tmp1;
	ValaBasicBlock* condition_block;
	gboolean has_default_label;
	GeeList* _tmp3;
	gboolean _tmp4;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_SWITCH_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	after_switch_block = vala_basic_block_new ();
	_tmp0 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp0 = vala_cfg_builder_jump_target_new_break_target (after_switch_block)));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	/* condition*/
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (vala_switch_statement_get_expression (stmt)));
	_tmp1 = NULL;
	condition_block = (_tmp1 = self->priv->current_block, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_switch_statement_get_expression (stmt)));
	has_default_label = FALSE;
	{
		GeeCollection* section_collection;
		GeeIterator* section_it;
		section_collection = vala_switch_statement_get_sections (stmt);
		section_it = gee_iterable_iterator (GEE_ITERABLE (section_collection));
		while (gee_iterator_next (section_it)) {
			ValaSwitchSection* section;
			section = ((ValaSwitchSection*) gee_iterator_get (section_it));
			{
				ValaBasicBlock* _tmp2;
				_tmp2 = NULL;
				self->priv->current_block = (_tmp2 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
				vala_basic_block_connect (condition_block, self->priv->current_block);
				{
					GeeCollection* stmt_collection;
					GeeIterator* stmt_it;
					stmt_collection = vala_block_get_statements (VALA_BLOCK (section));
					stmt_it = gee_iterable_iterator (GEE_ITERABLE (stmt_collection));
					while (gee_iterator_next (stmt_it)) {
						ValaStatement* stmt;
						stmt = ((ValaStatement*) gee_iterator_get (stmt_it));
						{
							vala_code_node_accept (VALA_CODE_NODE (stmt), VALA_CODE_VISITOR (self));
							(stmt == NULL ? NULL : (stmt = (g_object_unref (stmt), NULL)));
						}
					}
					(stmt_collection == NULL ? NULL : (stmt_collection = (g_object_unref (stmt_collection), NULL)));
					(stmt_it == NULL ? NULL : (stmt_it = (g_object_unref (stmt_it), NULL)));
				}
				if (vala_switch_section_has_default_label (section)) {
					has_default_label = TRUE;
				}
				if (self->priv->current_block != NULL) {
					/* end of switch section reachable
					 we don't allow fall-through*/
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (section)), "missing break statement at end of switch section");
					vala_code_node_set_error (VALA_CODE_NODE (section), TRUE);
					vala_basic_block_connect (self->priv->current_block, after_switch_block);
				}
				(section == NULL ? NULL : (section = (g_object_unref (section), NULL)));
			}
		}
		(section_collection == NULL ? NULL : (section_collection = (g_object_unref (section_collection), NULL)));
		(section_it == NULL ? NULL : (section_it = (g_object_unref (section_it), NULL)));
	}
	/* after switch
	 reachable?*/
	_tmp3 = NULL;
	if ((_tmp4 = !has_default_label || gee_collection_get_size (GEE_COLLECTION ((_tmp3 = vala_basic_block_get_predecessors (after_switch_block)))) > 0, (_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL))), _tmp4)) {
		ValaBasicBlock* _tmp6;
		ValaBasicBlock* _tmp5;
		_tmp6 = NULL;
		_tmp5 = NULL;
		self->priv->current_block = (_tmp6 = (_tmp5 = after_switch_block, (_tmp5 == NULL ? NULL : g_object_ref (_tmp5))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp6);
	} else {
		ValaBasicBlock* _tmp7;
		_tmp7 = NULL;
		self->priv->current_block = (_tmp7 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp7);
		self->priv->unreachable_reported = FALSE;
	}
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	(after_switch_block == NULL ? NULL : (after_switch_block = (g_object_unref (after_switch_block), NULL)));
	(condition_block == NULL ? NULL : (condition_block = (g_object_unref (condition_block), NULL)));
}


static void vala_cfg_builder_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* condition_block;
	ValaCFGBuilderJumpTarget* _tmp0;
	ValaBasicBlock* after_loop_block;
	ValaCFGBuilderJumpTarget* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* last_block;
	ValaBasicBlock* _tmp4;
	ValaBasicBlock* _tmp3;
	ValaBasicBlock* _tmp5;
	ValaBasicBlock* _tmp7;
	ValaBasicBlock* _tmp6;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	condition_block = vala_basic_block_new ();
	_tmp0 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp0 = vala_cfg_builder_jump_target_new_continue_target (condition_block)));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	after_loop_block = vala_basic_block_new ();
	_tmp1 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp1 = vala_cfg_builder_jump_target_new_break_target (after_loop_block)));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	/* condition*/
	_tmp2 = NULL;
	last_block = (_tmp2 = self->priv->current_block, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	vala_basic_block_connect (last_block, condition_block);
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->priv->current_block = (_tmp4 = (_tmp3 = condition_block, (_tmp3 == NULL ? NULL : g_object_ref (_tmp3))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp4);
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (vala_while_statement_get_condition (stmt)));
	vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_while_statement_get_condition (stmt)));
	/* loop block*/
	_tmp5 = NULL;
	self->priv->current_block = (_tmp5 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp5);
	vala_basic_block_connect (condition_block, self->priv->current_block);
	vala_code_node_accept (VALA_CODE_NODE (vala_while_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
	/* end of loop block reachable?*/
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, condition_block);
	}
	/* after loop*/
	vala_basic_block_connect (condition_block, after_loop_block);
	_tmp7 = NULL;
	_tmp6 = NULL;
	self->priv->current_block = (_tmp7 = (_tmp6 = after_loop_block, (_tmp6 == NULL ? NULL : g_object_ref (_tmp6))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp7);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	(condition_block == NULL ? NULL : (condition_block = (g_object_unref (condition_block), NULL)));
	(after_loop_block == NULL ? NULL : (after_loop_block = (g_object_unref (after_loop_block), NULL)));
	(last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL)));
}


static void vala_cfg_builder_real_visit_do_statement (ValaCodeVisitor* base, ValaDoStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* condition_block;
	ValaCFGBuilderJumpTarget* _tmp0;
	ValaBasicBlock* after_loop_block;
	ValaCFGBuilderJumpTarget* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* last_block;
	ValaBasicBlock* loop_block;
	ValaBasicBlock* _tmp4;
	ValaBasicBlock* _tmp3;
	GeeList* _tmp5;
	gboolean _tmp6;
	GeeList* _tmp11;
	gboolean _tmp12;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_DO_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	condition_block = vala_basic_block_new ();
	_tmp0 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp0 = vala_cfg_builder_jump_target_new_continue_target (condition_block)));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	after_loop_block = vala_basic_block_new ();
	_tmp1 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp1 = vala_cfg_builder_jump_target_new_break_target (after_loop_block)));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	/* loop block*/
	_tmp2 = NULL;
	last_block = (_tmp2 = self->priv->current_block, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	loop_block = vala_basic_block_new ();
	vala_basic_block_connect (last_block, loop_block);
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->priv->current_block = (_tmp4 = (_tmp3 = loop_block, (_tmp3 == NULL ? NULL : g_object_ref (_tmp3))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp4);
	vala_code_node_accept (VALA_CODE_NODE (vala_do_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
	/* condition
	 reachable?*/
	_tmp5 = NULL;
	if ((_tmp6 = self->priv->current_block != NULL || gee_collection_get_size (GEE_COLLECTION ((_tmp5 = vala_basic_block_get_predecessors (condition_block)))) > 0, (_tmp5 == NULL ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL))), _tmp6)) {
		ValaBasicBlock* _tmp10;
		ValaBasicBlock* _tmp9;
		if (self->priv->current_block != NULL) {
			ValaBasicBlock* _tmp8;
			ValaBasicBlock* _tmp7;
			_tmp8 = NULL;
			_tmp7 = NULL;
			last_block = (_tmp8 = (_tmp7 = self->priv->current_block, (_tmp7 == NULL ? NULL : g_object_ref (_tmp7))), (last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL))), _tmp8);
			vala_basic_block_connect (last_block, condition_block);
		}
		vala_basic_block_add_node (condition_block, VALA_CODE_NODE (vala_do_statement_get_condition (stmt)));
		vala_basic_block_connect (condition_block, loop_block);
		_tmp10 = NULL;
		_tmp9 = NULL;
		self->priv->current_block = (_tmp10 = (_tmp9 = condition_block, (_tmp9 == NULL ? NULL : g_object_ref (_tmp9))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp10);
		vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_do_statement_get_condition (stmt)));
	}
	/* after loop
	 reachable?*/
	_tmp11 = NULL;
	if ((_tmp12 = self->priv->current_block != NULL || gee_collection_get_size (GEE_COLLECTION ((_tmp11 = vala_basic_block_get_predecessors (after_loop_block)))) > 0, (_tmp11 == NULL ? NULL : (_tmp11 = (g_object_unref (_tmp11), NULL))), _tmp12)) {
		ValaBasicBlock* _tmp16;
		ValaBasicBlock* _tmp15;
		if (self->priv->current_block != NULL) {
			ValaBasicBlock* _tmp14;
			ValaBasicBlock* _tmp13;
			_tmp14 = NULL;
			_tmp13 = NULL;
			last_block = (_tmp14 = (_tmp13 = self->priv->current_block, (_tmp13 == NULL ? NULL : g_object_ref (_tmp13))), (last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL))), _tmp14);
			vala_basic_block_connect (last_block, after_loop_block);
		}
		_tmp16 = NULL;
		_tmp15 = NULL;
		self->priv->current_block = (_tmp16 = (_tmp15 = after_loop_block, (_tmp15 == NULL ? NULL : g_object_ref (_tmp15))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp16);
	}
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	(condition_block == NULL ? NULL : (condition_block = (g_object_unref (condition_block), NULL)));
	(after_loop_block == NULL ? NULL : (after_loop_block = (g_object_unref (after_loop_block), NULL)));
	(last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL)));
	(loop_block == NULL ? NULL : (loop_block = (g_object_unref (loop_block), NULL)));
}


static void vala_cfg_builder_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* iterator_block;
	ValaCFGBuilderJumpTarget* _tmp0;
	ValaBasicBlock* after_loop_block;
	ValaCFGBuilderJumpTarget* _tmp1;
	ValaBasicBlock* condition_block;
	ValaBasicBlock* _tmp3;
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* _tmp4;
	GeeList* _tmp5;
	gboolean _tmp6;
	ValaBasicBlock* _tmp10;
	ValaBasicBlock* _tmp9;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	/* initializer*/
	{
		GeeCollection* init_expr_collection;
		GeeIterator* init_expr_it;
		init_expr_collection = vala_for_statement_get_initializer (stmt);
		init_expr_it = gee_iterable_iterator (GEE_ITERABLE (init_expr_collection));
		while (gee_iterator_next (init_expr_it)) {
			ValaExpression* init_expr;
			init_expr = ((ValaExpression*) gee_iterator_get (init_expr_it));
			{
				vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (init_expr));
				vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (init_expr));
				(init_expr == NULL ? NULL : (init_expr = (g_object_unref (init_expr), NULL)));
			}
		}
		(init_expr_collection == NULL ? NULL : (init_expr_collection = (g_object_unref (init_expr_collection), NULL)));
		(init_expr_it == NULL ? NULL : (init_expr_it = (g_object_unref (init_expr_it), NULL)));
	}
	iterator_block = vala_basic_block_new ();
	_tmp0 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp0 = vala_cfg_builder_jump_target_new_continue_target (iterator_block)));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	after_loop_block = vala_basic_block_new ();
	_tmp1 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp1 = vala_cfg_builder_jump_target_new_break_target (after_loop_block)));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	/* condition*/
	condition_block = vala_basic_block_new ();
	vala_basic_block_connect (self->priv->current_block, condition_block);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->current_block = (_tmp3 = (_tmp2 = condition_block, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp3);
	if (vala_for_statement_get_condition (stmt) != NULL) {
		vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (vala_for_statement_get_condition (stmt)));
	}
	if (vala_for_statement_get_condition (stmt) != NULL) {
		vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_for_statement_get_condition (stmt)));
	}
	/* loop block*/
	_tmp4 = NULL;
	self->priv->current_block = (_tmp4 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp4);
	vala_basic_block_connect (condition_block, self->priv->current_block);
	vala_code_node_accept (VALA_CODE_NODE (vala_for_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
	/* iterator
	 reachable?*/
	_tmp5 = NULL;
	if ((_tmp6 = self->priv->current_block != NULL || gee_collection_get_size (GEE_COLLECTION ((_tmp5 = vala_basic_block_get_predecessors (iterator_block)))) > 0, (_tmp5 == NULL ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL))), _tmp6)) {
		ValaBasicBlock* _tmp8;
		ValaBasicBlock* _tmp7;
		if (self->priv->current_block != NULL) {
			vala_basic_block_connect (self->priv->current_block, iterator_block);
		}
		_tmp8 = NULL;
		_tmp7 = NULL;
		self->priv->current_block = (_tmp8 = (_tmp7 = iterator_block, (_tmp7 == NULL ? NULL : g_object_ref (_tmp7))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp8);
		{
			GeeCollection* it_expr_collection;
			GeeIterator* it_expr_it;
			it_expr_collection = vala_for_statement_get_iterator (stmt);
			it_expr_it = gee_iterable_iterator (GEE_ITERABLE (it_expr_collection));
			while (gee_iterator_next (it_expr_it)) {
				ValaExpression* it_expr;
				it_expr = ((ValaExpression*) gee_iterator_get (it_expr_it));
				{
					vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (it_expr));
					vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (it_expr));
					(it_expr == NULL ? NULL : (it_expr = (g_object_unref (it_expr), NULL)));
				}
			}
			(it_expr_collection == NULL ? NULL : (it_expr_collection = (g_object_unref (it_expr_collection), NULL)));
			(it_expr_it == NULL ? NULL : (it_expr_it = (g_object_unref (it_expr_it), NULL)));
		}
		vala_basic_block_connect (self->priv->current_block, condition_block);
	}
	/* after loop*/
	vala_basic_block_connect (condition_block, after_loop_block);
	_tmp10 = NULL;
	_tmp9 = NULL;
	self->priv->current_block = (_tmp10 = (_tmp9 = after_loop_block, (_tmp9 == NULL ? NULL : g_object_ref (_tmp9))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp10);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	(iterator_block == NULL ? NULL : (iterator_block = (g_object_unref (iterator_block), NULL)));
	(after_loop_block == NULL ? NULL : (after_loop_block = (g_object_unref (after_loop_block), NULL)));
	(condition_block == NULL ? NULL : (condition_block = (g_object_unref (condition_block), NULL)));
}


static void vala_cfg_builder_real_visit_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* loop_block;
	ValaCFGBuilderJumpTarget* _tmp0;
	ValaBasicBlock* after_loop_block;
	ValaCFGBuilderJumpTarget* _tmp1;
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* last_block;
	ValaBasicBlock* _tmp4;
	ValaBasicBlock* _tmp3;
	ValaBasicBlock* _tmp6;
	ValaBasicBlock* _tmp5;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	/* collection*/
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)));
	vala_cfg_builder_handle_errors (self, VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)));
	loop_block = vala_basic_block_new ();
	_tmp0 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp0 = vala_cfg_builder_jump_target_new_continue_target (loop_block)));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	after_loop_block = vala_basic_block_new ();
	_tmp1 = NULL;
	gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp1 = vala_cfg_builder_jump_target_new_break_target (after_loop_block)));
	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
	/* loop block*/
	_tmp2 = NULL;
	last_block = (_tmp2 = self->priv->current_block, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	vala_basic_block_connect (last_block, loop_block);
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->priv->current_block = (_tmp4 = (_tmp3 = loop_block, (_tmp3 == NULL ? NULL : g_object_ref (_tmp3))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp4);
	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, loop_block);
	}
	/* after loop*/
	vala_basic_block_connect (last_block, after_loop_block);
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, after_loop_block);
	}
	_tmp6 = NULL;
	_tmp5 = NULL;
	self->priv->current_block = (_tmp6 = (_tmp5 = after_loop_block, (_tmp5 == NULL ? NULL : g_object_ref (_tmp5))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp6);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	(loop_block == NULL ? NULL : (loop_block = (g_object_unref (loop_block), NULL)));
	(after_loop_block == NULL ? NULL : (after_loop_block = (g_object_unref (after_loop_block), NULL)));
	(last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL)));
}


static void vala_cfg_builder_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_BREAK_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
		for (; i >= 0; i--) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
			if (vala_cfg_builder_jump_target_get_break_target (jump_target)) {
				ValaBasicBlock* _tmp0;
				vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
				_tmp0 = NULL;
				self->priv->current_block = (_tmp0 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp0);
				self->priv->unreachable_reported = FALSE;
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
				return;
			} else {
				if (vala_cfg_builder_jump_target_get_finally_clause (jump_target)) {
					ValaBasicBlock* _tmp2;
					ValaBasicBlock* _tmp1;
					vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
					_tmp2 = NULL;
					_tmp1 = NULL;
					self->priv->current_block = (_tmp2 = (_tmp1 = vala_cfg_builder_jump_target_get_last_block (jump_target), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
				}
			}
			(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
		}
	}
	vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "no enclosing loop or switch statement found");
	vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
}


static void vala_cfg_builder_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_CONTINUE_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
		for (; i >= 0; i--) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
			if (vala_cfg_builder_jump_target_get_continue_target (jump_target)) {
				ValaBasicBlock* _tmp0;
				vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
				_tmp0 = NULL;
				self->priv->current_block = (_tmp0 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp0);
				self->priv->unreachable_reported = FALSE;
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
				return;
			} else {
				if (vala_cfg_builder_jump_target_get_finally_clause (jump_target)) {
					ValaBasicBlock* _tmp2;
					ValaBasicBlock* _tmp1;
					vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
					_tmp2 = NULL;
					_tmp1 = NULL;
					self->priv->current_block = (_tmp2 = (_tmp1 = vala_cfg_builder_jump_target_get_last_block (jump_target), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
				}
			}
			(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
		}
	}
	vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "no enclosing loop found");
	vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
}


static void vala_cfg_builder_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
		for (; i >= 0; i--) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
			if (vala_cfg_builder_jump_target_get_return_target (jump_target)) {
				ValaBasicBlock* _tmp0;
				vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
				_tmp0 = NULL;
				self->priv->current_block = (_tmp0 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp0);
				self->priv->unreachable_reported = FALSE;
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
				return;
			} else {
				if (vala_cfg_builder_jump_target_get_finally_clause (jump_target)) {
					ValaBasicBlock* _tmp2;
					ValaBasicBlock* _tmp1;
					vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
					_tmp2 = NULL;
					_tmp1 = NULL;
					self->priv->current_block = (_tmp2 = (_tmp1 = vala_cfg_builder_jump_target_get_last_block (jump_target), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
				}
			}
			(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
		}
	}
	vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "no enclosing loop found");
	vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
}


static void vala_cfg_builder_handle_errors (ValaCFGBuilder* self, ValaCodeNode* node) {
	g_return_if_fail (VALA_IS_CFG_BUILDER (self));
	g_return_if_fail (VALA_IS_CODE_NODE (node));
	if (vala_code_node_get_tree_can_fail (node)) {
		ValaBasicBlock* _tmp0;
		ValaBasicBlock* last_block;
		ValaBasicBlock* _tmp5;
		_tmp0 = NULL;
		last_block = (_tmp0 = self->priv->current_block, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
		{
			gint i;
			/* exceptional control flow*/
			i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
			for (; i >= 0; i--) {
				ValaCFGBuilderJumpTarget* jump_target;
				jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
				if (vala_cfg_builder_jump_target_get_return_target (jump_target)) {
					ValaBasicBlock* _tmp1;
					vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
					_tmp1 = NULL;
					self->priv->current_block = (_tmp1 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp1);
					self->priv->unreachable_reported = FALSE;
					(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
					break;
				} else {
					if (vala_cfg_builder_jump_target_get_error_target (jump_target)) {
						/* TODO check whether jump target catches node.error_type*/
						vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
						if (vala_cfg_builder_jump_target_get_error_domain (jump_target) == NULL) {
							ValaBasicBlock* _tmp2;
							/* catch all clause*/
							_tmp2 = NULL;
							self->priv->current_block = (_tmp2 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
							self->priv->unreachable_reported = FALSE;
							(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
							break;
						}
					} else {
						if (vala_cfg_builder_jump_target_get_finally_clause (jump_target)) {
							ValaBasicBlock* _tmp4;
							ValaBasicBlock* _tmp3;
							vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
							_tmp4 = NULL;
							_tmp3 = NULL;
							self->priv->current_block = (_tmp4 = (_tmp3 = vala_cfg_builder_jump_target_get_last_block (jump_target), (_tmp3 == NULL ? NULL : g_object_ref (_tmp3))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp4);
						}
					}
				}
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
			}
		}
		/* normal control flow*/
		_tmp5 = NULL;
		self->priv->current_block = (_tmp5 = vala_basic_block_new (), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp5);
		vala_basic_block_connect (last_block, self->priv->current_block);
		(last_block == NULL ? NULL : (last_block = (g_object_unref (last_block), NULL)));
	}
}


static void vala_cfg_builder_real_visit_throw_statement (ValaCodeVisitor* base, ValaThrowStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_THROW_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, VALA_CODE_NODE (stmt));
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
		for (; i >= 0; i--) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
			if (vala_cfg_builder_jump_target_get_return_target (jump_target)) {
				ValaBasicBlock* _tmp0;
				vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
				_tmp0 = NULL;
				self->priv->current_block = (_tmp0 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp0);
				self->priv->unreachable_reported = FALSE;
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
				return;
			} else {
				if (vala_cfg_builder_jump_target_get_error_target (jump_target)) {
					/* TODO check whether jump target catches stmt.error_type*/
					vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
					if (vala_cfg_builder_jump_target_get_error_domain (jump_target) == NULL) {
						ValaBasicBlock* _tmp1;
						_tmp1 = NULL;
						self->priv->current_block = (_tmp1 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp1);
						self->priv->unreachable_reported = FALSE;
						(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
						return;
					}
				} else {
					if (vala_cfg_builder_jump_target_get_finally_clause (jump_target)) {
						ValaBasicBlock* _tmp3;
						ValaBasicBlock* _tmp2;
						vala_basic_block_connect (self->priv->current_block, vala_cfg_builder_jump_target_get_basic_block (jump_target));
						_tmp3 = NULL;
						_tmp2 = NULL;
						self->priv->current_block = (_tmp3 = (_tmp2 = vala_cfg_builder_jump_target_get_last_block (jump_target), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp3);
					}
				}
			}
			(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
		}
	}
	g_assert_not_reached ();
}


static void vala_cfg_builder_real_visit_try_statement (ValaCodeVisitor* base, ValaTryStatement* stmt) {
	ValaCFGBuilder * self;
	ValaBasicBlock* after_try_block;
	ValaBasicBlock* finally_block;
	gint finally_jump_stack_size;
	GeeList* catch_clauses;
	GeeList* catch_stack;
	GeeList* _tmp17;
	gboolean _tmp18;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_TRY_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	after_try_block = vala_basic_block_new ();
	finally_block = NULL;
	if (vala_try_statement_get_finally_body (stmt) != NULL) {
		ValaBasicBlock* _tmp0;
		ValaBasicBlock* _tmp2;
		ValaBasicBlock* _tmp1;
		ValaCFGBuilderJumpTarget* _tmp3;
		_tmp0 = NULL;
		finally_block = (_tmp0 = vala_basic_block_new (), (finally_block == NULL ? NULL : (finally_block = (g_object_unref (finally_block), NULL))), _tmp0);
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->current_block = (_tmp2 = (_tmp1 = finally_block, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp2);
		vala_code_node_accept (VALA_CODE_NODE (vala_try_statement_get_finally_body (stmt)), VALA_CODE_VISITOR (self));
		if (self->priv->current_block == NULL) {
			/* don't allow finally blocks with e.g. return statements*/
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "end of finally block not reachable");
			vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
			(after_try_block == NULL ? NULL : (after_try_block = (g_object_unref (after_try_block), NULL)));
			(finally_block == NULL ? NULL : (finally_block = (g_object_unref (finally_block), NULL)));
			return;
		}
		_tmp3 = NULL;
		gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp3 = vala_cfg_builder_jump_target_new_finally_clause (finally_block, self->priv->current_block)));
		(_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL)));
	}
	finally_jump_stack_size = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack));
	catch_clauses = vala_try_statement_get_catch_clauses (stmt);
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (catch_clauses)) - 1;
		for (; i >= 0; i--) {
			ValaCatchClause* catch_clause;
			catch_clause = ((ValaCatchClause*) gee_list_get (((GeeList*) catch_clauses), i));
			if (vala_catch_clause_get_error_type (catch_clause) != NULL) {
				ValaCFGBuilderJumpTarget* _tmp6;
				ValaBasicBlock* _tmp4;
				ValaTypesymbol* _tmp5;
				_tmp6 = NULL;
				_tmp4 = NULL;
				_tmp5 = NULL;
				gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp6 = vala_cfg_builder_jump_target_new_error_target ((_tmp4 = vala_basic_block_new ()), catch_clause, (_tmp5 = vala_data_type_get_data_type (vala_catch_clause_get_error_type (catch_clause)), (VALA_IS_ERROR_DOMAIN (_tmp5) ? ((ValaErrorDomain*) _tmp5) : NULL)), NULL)));
				(_tmp6 == NULL ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL)));
				(_tmp4 == NULL ? NULL : (_tmp4 = (g_object_unref (_tmp4), NULL)));
			} else {
				ValaCFGBuilderJumpTarget* _tmp8;
				ValaBasicBlock* _tmp7;
				_tmp8 = NULL;
				_tmp7 = NULL;
				gee_collection_add (GEE_COLLECTION (self->priv->jump_stack), (_tmp8 = vala_cfg_builder_jump_target_new_error_target ((_tmp7 = vala_basic_block_new ()), catch_clause, NULL, NULL)));
				(_tmp8 == NULL ? NULL : (_tmp8 = (g_object_unref (_tmp8), NULL)));
				(_tmp7 == NULL ? NULL : (_tmp7 = (g_object_unref (_tmp7), NULL)));
			}
			(catch_clause == NULL ? NULL : (catch_clause = (g_object_unref (catch_clause), NULL)));
		}
	}
	vala_code_node_accept (VALA_CODE_NODE (vala_try_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
	if (self->priv->current_block != NULL) {
		if (finally_block != NULL) {
			ValaBasicBlock* _tmp10;
			ValaBasicBlock* _tmp9;
			vala_basic_block_connect (self->priv->current_block, finally_block);
			_tmp10 = NULL;
			_tmp9 = NULL;
			self->priv->current_block = (_tmp10 = (_tmp9 = finally_block, (_tmp9 == NULL ? NULL : g_object_ref (_tmp9))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp10);
		}
		vala_basic_block_connect (self->priv->current_block, after_try_block);
	}
	/* remove catch clauses from jump stack*/
	catch_stack = GEE_LIST (gee_array_list_new (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
	{
		gint i;
		i = gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1;
		for (; i >= finally_jump_stack_size; i--) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_list_get (((GeeList*) self->priv->jump_stack), i));
			gee_collection_add (GEE_COLLECTION (catch_stack), jump_target);
			gee_list_remove_at (self->priv->jump_stack, i);
			(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
		}
	}
	{
		GeeList* jump_target_collection;
		GeeIterator* jump_target_it;
		jump_target_collection = catch_stack;
		jump_target_it = gee_iterable_iterator (GEE_ITERABLE (jump_target_collection));
		while (gee_iterator_next (jump_target_it)) {
			ValaCFGBuilderJumpTarget* jump_target;
			jump_target = ((ValaCFGBuilderJumpTarget*) gee_iterator_get (jump_target_it));
			{
				GeeList* _tmp11;
				gboolean _tmp12;
				_tmp11 = NULL;
				if ((_tmp12 = gee_collection_get_size (GEE_COLLECTION ((_tmp11 = vala_basic_block_get_predecessors (vala_cfg_builder_jump_target_get_basic_block (jump_target))))) == 0, (_tmp11 == NULL ? NULL : (_tmp11 = (g_object_unref (_tmp11), NULL))), _tmp12)) {
					/* unreachable*/
					vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_cfg_builder_jump_target_get_catch_clause (jump_target))), "unreachable catch clause detected");
				} else {
					ValaBasicBlock* _tmp14;
					ValaBasicBlock* _tmp13;
					_tmp14 = NULL;
					_tmp13 = NULL;
					self->priv->current_block = (_tmp14 = (_tmp13 = vala_cfg_builder_jump_target_get_basic_block (jump_target), (_tmp13 == NULL ? NULL : g_object_ref (_tmp13))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp14);
					vala_code_node_accept (VALA_CODE_NODE (vala_catch_clause_get_body (vala_cfg_builder_jump_target_get_catch_clause (jump_target))), VALA_CODE_VISITOR (self));
					if (self->priv->current_block != NULL) {
						if (finally_block != NULL) {
							ValaBasicBlock* _tmp16;
							ValaBasicBlock* _tmp15;
							vala_basic_block_connect (self->priv->current_block, finally_block);
							_tmp16 = NULL;
							_tmp15 = NULL;
							self->priv->current_block = (_tmp16 = (_tmp15 = finally_block, (_tmp15 == NULL ? NULL : g_object_ref (_tmp15))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp16);
						}
						vala_basic_block_connect (self->priv->current_block, after_try_block);
					}
				}
				(jump_target == NULL ? NULL : (jump_target = (g_object_unref (jump_target), NULL)));
			}
		}
		(jump_target_it == NULL ? NULL : (jump_target_it = (g_object_unref (jump_target_it), NULL)));
	}
	if (finally_block != NULL) {
		gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size (GEE_COLLECTION (self->priv->jump_stack)) - 1);
	}
	/* after try statement
	 reachable?*/
	_tmp17 = NULL;
	if ((_tmp18 = gee_collection_get_size (GEE_COLLECTION ((_tmp17 = vala_basic_block_get_predecessors (after_try_block)))) > 0, (_tmp17 == NULL ? NULL : (_tmp17 = (g_object_unref (_tmp17), NULL))), _tmp18)) {
		ValaBasicBlock* _tmp20;
		ValaBasicBlock* _tmp19;
		_tmp20 = NULL;
		_tmp19 = NULL;
		self->priv->current_block = (_tmp20 = (_tmp19 = after_try_block, (_tmp19 == NULL ? NULL : g_object_ref (_tmp19))), (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp20);
	} else {
		ValaBasicBlock* _tmp21;
		_tmp21 = NULL;
		self->priv->current_block = (_tmp21 = NULL, (self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL))), _tmp21);
		self->priv->unreachable_reported = FALSE;
	}
	(after_try_block == NULL ? NULL : (after_try_block = (g_object_unref (after_try_block), NULL)));
	(finally_block == NULL ? NULL : (finally_block = (g_object_unref (finally_block), NULL)));
	(catch_clauses == NULL ? NULL : (catch_clauses = (g_object_unref (catch_clauses), NULL)));
	(catch_stack == NULL ? NULL : (catch_stack = (g_object_unref (catch_stack), NULL)));
}


static void vala_cfg_builder_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (base);
	g_return_if_fail (VALA_IS_LOCK_STATEMENT (stmt));
	if (vala_cfg_builder_unreachable (self, VALA_CODE_NODE (stmt))) {
		return;
	}
	vala_code_node_accept (VALA_CODE_NODE (vala_lock_statement_get_body (stmt)), VALA_CODE_VISITOR (self));
}


static gboolean vala_cfg_builder_unreachable (ValaCFGBuilder* self, ValaCodeNode* node) {
	g_return_val_if_fail (VALA_IS_CFG_BUILDER (self), FALSE);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), FALSE);
	if (self->priv->current_block == NULL) {
		if (!self->priv->unreachable_reported) {
			vala_report_warning (vala_code_node_get_source_reference (node), "unreachable code detected");
			self->priv->unreachable_reported = TRUE;
		}
		return TRUE;
	}
	return FALSE;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_break_target (ValaBasicBlock* basic_block) {
	ValaCFGBuilderJumpTarget * self;
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (basic_block), NULL);
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	vala_cfg_builder_jump_target_set_basic_block (self, basic_block);
	vala_cfg_builder_jump_target_set_break_target (self, TRUE);
	return self;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_continue_target (ValaBasicBlock* basic_block) {
	ValaCFGBuilderJumpTarget * self;
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (basic_block), NULL);
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	vala_cfg_builder_jump_target_set_basic_block (self, basic_block);
	vala_cfg_builder_jump_target_set_continue_target (self, TRUE);
	return self;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_return_target (ValaBasicBlock* basic_block) {
	ValaCFGBuilderJumpTarget * self;
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (basic_block), NULL);
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	vala_cfg_builder_jump_target_set_basic_block (self, basic_block);
	vala_cfg_builder_jump_target_set_return_target (self, TRUE);
	return self;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_error_target (ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code) {
	ValaCFGBuilderJumpTarget * self;
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (basic_block), NULL);
	g_return_val_if_fail (VALA_IS_CATCH_CLAUSE (catch_clause), NULL);
	g_return_val_if_fail (error_domain == NULL || VALA_IS_ERROR_DOMAIN (error_domain), NULL);
	g_return_val_if_fail (error_code == NULL || VALA_IS_ERROR_CODE (error_code), NULL);
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	vala_cfg_builder_jump_target_set_basic_block (self, basic_block);
	vala_cfg_builder_jump_target_set_catch_clause (self, catch_clause);
	vala_cfg_builder_jump_target_set_error_domain (self, error_domain);
	vala_cfg_builder_jump_target_set_error_code (self, error_code);
	vala_cfg_builder_jump_target_set_error_target (self, TRUE);
	return self;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new_finally_clause (ValaBasicBlock* basic_block, ValaBasicBlock* last_block) {
	ValaCFGBuilderJumpTarget * self;
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (basic_block), NULL);
	g_return_val_if_fail (VALA_IS_BASIC_BLOCK (last_block), NULL);
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	vala_cfg_builder_jump_target_set_basic_block (self, basic_block);
	vala_cfg_builder_jump_target_set_last_block (self, last_block);
	vala_cfg_builder_jump_target_set_finally_clause (self, TRUE);
	return self;
}


static ValaCFGBuilderJumpTarget* vala_cfg_builder_jump_target_new (void) {
	ValaCFGBuilderJumpTarget * self;
	self = g_object_newv (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, 0, NULL);
	return self;
}


static gboolean vala_cfg_builder_jump_target_get_break_target (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), FALSE);
	return self->priv->_break_target;
}


static void vala_cfg_builder_jump_target_set_break_target (ValaCFGBuilderJumpTarget* self, gboolean value) {
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	self->priv->_break_target = value;
}


static gboolean vala_cfg_builder_jump_target_get_continue_target (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), FALSE);
	return self->priv->_continue_target;
}


static void vala_cfg_builder_jump_target_set_continue_target (ValaCFGBuilderJumpTarget* self, gboolean value) {
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	self->priv->_continue_target = value;
}


static gboolean vala_cfg_builder_jump_target_get_return_target (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), FALSE);
	return self->priv->_return_target;
}


static void vala_cfg_builder_jump_target_set_return_target (ValaCFGBuilderJumpTarget* self, gboolean value) {
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	self->priv->_return_target = value;
}


static gboolean vala_cfg_builder_jump_target_get_error_target (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), FALSE);
	return self->priv->_error_target;
}


static void vala_cfg_builder_jump_target_set_error_target (ValaCFGBuilderJumpTarget* self, gboolean value) {
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	self->priv->_error_target = value;
}


static ValaErrorDomain* vala_cfg_builder_jump_target_get_error_domain (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), NULL);
	return self->priv->_error_domain;
}


static void vala_cfg_builder_jump_target_set_error_domain (ValaCFGBuilderJumpTarget* self, ValaErrorDomain* value) {
	ValaErrorDomain* _tmp2;
	ValaErrorDomain* _tmp1;
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_error_domain = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_error_domain == NULL ? NULL : (self->priv->_error_domain = (g_object_unref (self->priv->_error_domain), NULL))), _tmp2);
}


static ValaErrorCode* vala_cfg_builder_jump_target_get_error_code (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), NULL);
	return self->priv->_error_code;
}


static void vala_cfg_builder_jump_target_set_error_code (ValaCFGBuilderJumpTarget* self, ValaErrorCode* value) {
	ValaErrorCode* _tmp2;
	ValaErrorCode* _tmp1;
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_error_code = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_error_code == NULL ? NULL : (self->priv->_error_code = (g_object_unref (self->priv->_error_code), NULL))), _tmp2);
}


static gboolean vala_cfg_builder_jump_target_get_finally_clause (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), FALSE);
	return self->priv->_finally_clause;
}


static void vala_cfg_builder_jump_target_set_finally_clause (ValaCFGBuilderJumpTarget* self, gboolean value) {
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	self->priv->_finally_clause = value;
}


static ValaBasicBlock* vala_cfg_builder_jump_target_get_basic_block (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), NULL);
	return self->priv->_basic_block;
}


static void vala_cfg_builder_jump_target_set_basic_block (ValaCFGBuilderJumpTarget* self, ValaBasicBlock* value) {
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* _tmp1;
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_basic_block = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_basic_block == NULL ? NULL : (self->priv->_basic_block = (g_object_unref (self->priv->_basic_block), NULL))), _tmp2);
}


static ValaBasicBlock* vala_cfg_builder_jump_target_get_last_block (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), NULL);
	return self->priv->_last_block;
}


static void vala_cfg_builder_jump_target_set_last_block (ValaCFGBuilderJumpTarget* self, ValaBasicBlock* value) {
	ValaBasicBlock* _tmp2;
	ValaBasicBlock* _tmp1;
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_last_block = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_last_block == NULL ? NULL : (self->priv->_last_block = (g_object_unref (self->priv->_last_block), NULL))), _tmp2);
}


static ValaCatchClause* vala_cfg_builder_jump_target_get_catch_clause (ValaCFGBuilderJumpTarget* self) {
	g_return_val_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self), NULL);
	return self->priv->_catch_clause;
}


static void vala_cfg_builder_jump_target_set_catch_clause (ValaCFGBuilderJumpTarget* self, ValaCatchClause* value) {
	ValaCatchClause* _tmp2;
	ValaCatchClause* _tmp1;
	g_return_if_fail (VALA_CFG_BUILDER_IS_JUMP_TARGET (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_catch_clause = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_catch_clause == NULL ? NULL : (self->priv->_catch_clause = (g_object_unref (self->priv->_catch_clause), NULL))), _tmp2);
}


static void vala_cfg_builder_jump_target_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaCFGBuilderJumpTarget * self;
	self = VALA_CFG_BUILDER_JUMP_TARGET (object);
	switch (property_id) {
		case VALA_CFG_BUILDER_JUMP_TARGET_BREAK_TARGET:
		g_value_set_boolean (value, vala_cfg_builder_jump_target_get_break_target (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_CONTINUE_TARGET:
		g_value_set_boolean (value, vala_cfg_builder_jump_target_get_continue_target (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_RETURN_TARGET:
		g_value_set_boolean (value, vala_cfg_builder_jump_target_get_return_target (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_TARGET:
		g_value_set_boolean (value, vala_cfg_builder_jump_target_get_error_target (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_DOMAIN:
		g_value_set_object (value, vala_cfg_builder_jump_target_get_error_domain (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_CODE:
		g_value_set_object (value, vala_cfg_builder_jump_target_get_error_code (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_FINALLY_CLAUSE:
		g_value_set_boolean (value, vala_cfg_builder_jump_target_get_finally_clause (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_BASIC_BLOCK:
		g_value_set_object (value, vala_cfg_builder_jump_target_get_basic_block (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_LAST_BLOCK:
		g_value_set_object (value, vala_cfg_builder_jump_target_get_last_block (self));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_CATCH_CLAUSE:
		g_value_set_object (value, vala_cfg_builder_jump_target_get_catch_clause (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_cfg_builder_jump_target_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaCFGBuilderJumpTarget * self;
	self = VALA_CFG_BUILDER_JUMP_TARGET (object);
	switch (property_id) {
		case VALA_CFG_BUILDER_JUMP_TARGET_BREAK_TARGET:
		vala_cfg_builder_jump_target_set_break_target (self, g_value_get_boolean (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_CONTINUE_TARGET:
		vala_cfg_builder_jump_target_set_continue_target (self, g_value_get_boolean (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_RETURN_TARGET:
		vala_cfg_builder_jump_target_set_return_target (self, g_value_get_boolean (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_TARGET:
		vala_cfg_builder_jump_target_set_error_target (self, g_value_get_boolean (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_DOMAIN:
		vala_cfg_builder_jump_target_set_error_domain (self, g_value_get_object (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_ERROR_CODE:
		vala_cfg_builder_jump_target_set_error_code (self, g_value_get_object (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_FINALLY_CLAUSE:
		vala_cfg_builder_jump_target_set_finally_clause (self, g_value_get_boolean (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_BASIC_BLOCK:
		vala_cfg_builder_jump_target_set_basic_block (self, g_value_get_object (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_LAST_BLOCK:
		vala_cfg_builder_jump_target_set_last_block (self, g_value_get_object (value));
		break;
		case VALA_CFG_BUILDER_JUMP_TARGET_CATCH_CLAUSE:
		vala_cfg_builder_jump_target_set_catch_clause (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_cfg_builder_jump_target_class_init (ValaCFGBuilderJumpTargetClass * klass) {
	vala_cfg_builder_jump_target_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCFGBuilderJumpTargetPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_cfg_builder_jump_target_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_cfg_builder_jump_target_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_cfg_builder_jump_target_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_BREAK_TARGET, g_param_spec_boolean ("break-target", "break-target", "break-target", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_CONTINUE_TARGET, g_param_spec_boolean ("continue-target", "continue-target", "continue-target", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_RETURN_TARGET, g_param_spec_boolean ("return-target", "return-target", "return-target", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_ERROR_TARGET, g_param_spec_boolean ("error-target", "error-target", "error-target", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_ERROR_DOMAIN, g_param_spec_object ("error-domain", "error-domain", "error-domain", VALA_TYPE_ERROR_DOMAIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_ERROR_CODE, g_param_spec_object ("error-code", "error-code", "error-code", VALA_TYPE_ERROR_CODE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_FINALLY_CLAUSE, g_param_spec_boolean ("finally-clause", "finally-clause", "finally-clause", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_BASIC_BLOCK, g_param_spec_object ("basic-block", "basic-block", "basic-block", VALA_TYPE_BASIC_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_LAST_BLOCK, g_param_spec_object ("last-block", "last-block", "last-block", VALA_TYPE_BASIC_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CFG_BUILDER_JUMP_TARGET_CATCH_CLAUSE, g_param_spec_object ("catch-clause", "catch-clause", "catch-clause", VALA_TYPE_CATCH_CLAUSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_cfg_builder_jump_target_init (ValaCFGBuilderJumpTarget * self) {
	self->priv = VALA_CFG_BUILDER_JUMP_TARGET_GET_PRIVATE (self);
}


static void vala_cfg_builder_jump_target_dispose (GObject * obj) {
	ValaCFGBuilderJumpTarget * self;
	self = VALA_CFG_BUILDER_JUMP_TARGET (obj);
	(self->priv->_error_domain == NULL ? NULL : (self->priv->_error_domain = (g_object_unref (self->priv->_error_domain), NULL)));
	(self->priv->_error_code == NULL ? NULL : (self->priv->_error_code = (g_object_unref (self->priv->_error_code), NULL)));
	(self->priv->_basic_block == NULL ? NULL : (self->priv->_basic_block = (g_object_unref (self->priv->_basic_block), NULL)));
	(self->priv->_last_block == NULL ? NULL : (self->priv->_last_block = (g_object_unref (self->priv->_last_block), NULL)));
	(self->priv->_catch_clause == NULL ? NULL : (self->priv->_catch_clause = (g_object_unref (self->priv->_catch_clause), NULL)));
	G_OBJECT_CLASS (vala_cfg_builder_jump_target_parent_class)->dispose (obj);
}


static GType vala_cfg_builder_jump_target_get_type (void) {
	static GType vala_cfg_builder_jump_target_type_id = 0;
	if (G_UNLIKELY (vala_cfg_builder_jump_target_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCFGBuilderJumpTargetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_cfg_builder_jump_target_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCFGBuilderJumpTarget), 0, (GInstanceInitFunc) vala_cfg_builder_jump_target_init };
		vala_cfg_builder_jump_target_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaCFGBuilderJumpTarget", &g_define_type_info, 0);
	}
	return vala_cfg_builder_jump_target_type_id;
}


static void vala_cfg_builder_class_init (ValaCFGBuilderClass * klass) {
	vala_cfg_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCFGBuilderPrivate));
	G_OBJECT_CLASS (klass)->dispose = vala_cfg_builder_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = vala_cfg_builder_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vala_cfg_builder_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = vala_cfg_builder_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = vala_cfg_builder_real_visit_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum = vala_cfg_builder_real_visit_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_domain = vala_cfg_builder_real_visit_error_domain;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vala_cfg_builder_real_visit_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property = vala_cfg_builder_real_visit_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property_accessor = vala_cfg_builder_real_visit_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_block = vala_cfg_builder_real_visit_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_declaration_statement = vala_cfg_builder_real_visit_declaration_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_cfg_builder_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_cfg_builder_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_switch_statement = vala_cfg_builder_real_visit_switch_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_cfg_builder_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_do_statement = vala_cfg_builder_real_visit_do_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_cfg_builder_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_foreach_statement = vala_cfg_builder_real_visit_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_break_statement = vala_cfg_builder_real_visit_break_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_continue_statement = vala_cfg_builder_real_visit_continue_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_cfg_builder_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_throw_statement = vala_cfg_builder_real_visit_throw_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_try_statement = vala_cfg_builder_real_visit_try_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_lock_statement = vala_cfg_builder_real_visit_lock_statement;
}


static void vala_cfg_builder_init (ValaCFGBuilder * self) {
	self->priv = VALA_CFG_BUILDER_GET_PRIVATE (self);
	self->priv->jump_stack = GEE_LIST (gee_array_list_new (VALA_CFG_BUILDER_TYPE_JUMP_TARGET, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
}


static void vala_cfg_builder_dispose (GObject * obj) {
	ValaCFGBuilder * self;
	self = VALA_CFG_BUILDER (obj);
	(self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL)));
	(self->priv->current_block == NULL ? NULL : (self->priv->current_block = (g_object_unref (self->priv->current_block), NULL)));
	(self->priv->jump_stack == NULL ? NULL : (self->priv->jump_stack = (g_object_unref (self->priv->jump_stack), NULL)));
	G_OBJECT_CLASS (vala_cfg_builder_parent_class)->dispose (obj);
}


GType vala_cfg_builder_get_type (void) {
	static GType vala_cfg_builder_type_id = 0;
	if (G_UNLIKELY (vala_cfg_builder_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCFGBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_cfg_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCFGBuilder), 0, (GInstanceInitFunc) vala_cfg_builder_init };
		vala_cfg_builder_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaCFGBuilder", &g_define_type_info, 0);
	}
	return vala_cfg_builder_type_id;
}




